@@ -1,5 +1,5 @@
 /*
- * (C) Gražvydas "notaz" Ignotas, 2006-2010
+ * (C) Gražvydas "notaz" Ignotas, 2006-2011
  *
  * This work is licensed under the terms of any of these licenses
  * (at your option):
@@ -372,6 +372,28 @@ static int me_count(const menu_entry *ent)
 	return ret;
 }
 
+static unsigned int me_read_onoff(const menu_entry *ent)
+{
+	// guess var size based on mask to avoid reading too much
+	if (ent->mask & 0xffff0000)
+		return *(unsigned int *)ent->var & ent->mask;
+	else if (ent->mask & 0xff00)
+		return *(unsigned short *)ent->var & ent->mask;
+	else
+		return *(unsigned char *)ent->var & ent->mask;
+}
+
+static void me_toggle_onoff(menu_entry *ent)
+{
+	// guess var size based on mask to avoid reading too much
+	if (ent->mask & 0xffff0000)
+		*(unsigned int *)ent->var ^= ent->mask;
+	else if (ent->mask & 0xff00)
+		*(unsigned short *)ent->var ^= ent->mask;
+	else
+		*(unsigned char *)ent->var ^= ent->mask;
+}
+
 static void me_draw(const menu_entry *entries, int sel, void (*draw_more)(void))
 {
 	const menu_entry *ent, *ent_sel = entries;
@@ -477,7 +499,7 @@ static void me_draw(const menu_entry *entries, int sel, void (*draw_more)(void))
 		case MB_NONE:
 			break;
 		case MB_OPT_ONOFF:
-			text_out16(x + col2_offs, y, (*(int *)ent->var & ent->mask) ? "ON" : "OFF");
+			text_out16(x + col2_offs, y, me_read_onoff(ent) ? "ON" : "OFF");
 			break;
 		case MB_OPT_RANGE:
 			text_out16(x + col2_offs, y, "%i", *(int *)ent->var);
@@ -545,7 +567,7 @@ static int me_process(menu_entry *entry, int is_next, int is_lr)
 	{
 		case MB_OPT_ONOFF:
 		case MB_OPT_CUSTONOFF:
-			*(int *)entry->var ^= entry->mask;
+			me_toggle_onoff(entry);
 			return 1;
 		case MB_OPT_RANGE:
 		case MB_OPT_CUSTRANGE:
@@ -560,11 +582,11 @@ static int me_process(menu_entry *entry, int is_next, int is_lr)
 			names = (const char **)entry->data;
 			for (c = 0; names[c] != NULL; c++)
 				;
-			*(int *)entry->var += is_next ? 1 : -1;
-			if (*(int *)entry->var < 0)
-				*(int *)entry->var = 0;
-			if (*(int *)entry->var >= c)
-				*(int *)entry->var = c - 1;
+			*(signed char *)entry->var += is_next ? 1 : -1;
+			if (*(signed char *)entry->var < 0)
+				*(signed char *)entry->var = 0;
+			if (*(signed char *)entry->var >= c)
+				*(signed char *)entry->var = c - 1;
 			return 1;
 		default:
 			return 0;
@@ -573,14 +595,14 @@ static int me_process(menu_entry *entry, int is_next, int is_lr)
 
 static void debug_menu_loop(void);
 
-static void me_loop(menu_entry *menu, int *menu_sel, void (*draw_more)(void))
+static int me_loop_d(menu_entry *menu, int *menu_sel, void (*draw_prep)(void), void (*draw_more)(void))
 {
-	int ret, inp, sel = *menu_sel, menu_sel_max;
+	int ret = 0, inp, sel = *menu_sel, menu_sel_max;
 
 	menu_sel_max = me_count(menu) - 1;
 	if (menu_sel_max < 0) {
 		lprintf("no enabled menu entries\n");
-		return;
+		return 0;
 	}
 
 	while ((!menu[sel].enabled || !menu[sel].selectable) && sel < menu_sel_max)
@@ -592,6 +614,9 @@ static void me_loop(menu_entry *menu, int *menu_sel, void (*draw_more)(void))
 
 	for (;;)
 	{
+		if (draw_prep != NULL)
+			draw_prep();
+
 		me_draw(menu, sel, draw_more);
 		inp = in_menu_wait(PBTN_UP|PBTN_DOWN|PBTN_LEFT|PBTN_RIGHT|
 					PBTN_MOK|PBTN_MBACK|PBTN_MENU|PBTN_L|PBTN_R, 70);
@@ -636,6 +661,13 @@ static void me_loop(menu_entry *menu, int *menu_sel, void (*draw_more)(void))
 		}
 	}
 	*menu_sel = sel;
+
+	return ret;
+}
+
+static int me_loop(menu_entry *menu, int *menu_sel)
+{
+	return me_loop_d(menu, menu_sel, NULL, NULL);
 }
 
 /* ***************************************** */
@@ -766,11 +798,6 @@ static int scandir_cmp(const void *p1, const void *p2)
 	return alphasort(d1, d2);
 }
 
-static const char *filter_exts[] = {
-	".mp3", ".MP3", ".srm", ".brm", "s.gz", ".mds",	"bcfg", ".txt", ".htm", "html",
-	".jpg", ".gpe"
-};
-
 static int scandir_filter(const struct dirent *ent)
 {
 	const char *p;
