@@ -20,21 +20,30 @@
 #include "plugin_lib.h"
 #include "pcnt.h"
 #include "menu.h"
+#include "plat.h"
 #include "../libpcsxcore/misc.h"
 #include "../libpcsxcore/new_dynarec/new_dynarec.h"
 #include "../plugins/cdrcimg/cdrcimg.h"
 #include "common/plat.h"
-#include "common/input.h"
 #include "common/readpng.h"
+#include "common/input.h"
+#include "linux/in_evdev.h"
 
 // don't include debug.h - it breaks ARM build (R1 redefined)
 void StartDebugger();
 void StopDebugger();
 
+// sound plugin
+extern int iUseReverb;
+extern int iUseInterpolation;
+extern int iXAPitch;
+extern int iSPUIRQWait;
+extern int iUseTimer;
+extern int iVolume;
+
 int ready_to_go;
 unsigned long gpuDisp;
 char cfgfile_basename[MAXPATHLEN];
-static char *(*real_getenv)(const char *name);
 int state_slot;
 enum sched_action emu_action, emu_action_old;
 char hud_msg[64];
@@ -72,7 +81,7 @@ static void CheckSubDir() {
 	create_profile_dir("/screenshots/");
 }
 
-int get_gameid_filename(char *buf, int size, const char *fmt, int i) {
+static int get_gameid_filename(char *buf, int size, const char *fmt, int i) {
 	char trimlabel[33];
 	int j;
 
@@ -124,11 +133,41 @@ static void set_default_paths(void)
 #if defined(__arm__) && !defined(__ARM_ARCH_7A__) /* XXX */
 	strcpy(Config.Gpu, "gpuPCSX4ALL.so");
 #endif
-	Config.PsxAuto = 1;
 
 	snprintf(Config.PatchesDir, sizeof(Config.PatchesDir), "." PATCHES_DIR);
 }
 
+void emu_set_default_config(void)
+{
+	// try to set sane config on which most games work
+	Config.Xa = Config.Cdda = Config.Sio =
+	Config.SpuIrq = Config.RCntFix = Config.VSyncWA = 0;
+	Config.CdrReschedule = 0;
+	Config.PsxAuto = 1;
+
+	pl_rearmed_cbs.gpu_peops.iUseDither = 0;
+	pl_rearmed_cbs.gpu_peops.dwActFixes = 1<<7;
+	pl_rearmed_cbs.gpu_unai.abe_hack =
+	pl_rearmed_cbs.gpu_unai.no_light =
+	pl_rearmed_cbs.gpu_unai.no_blend = 0;
+
+	iUseReverb = 2;
+	iUseInterpolation = 1;
+	iXAPitch = 0;
+	iSPUIRQWait = 1;
+	iUseTimer = 2;
+	iVolume = 768;
+#ifndef __ARM_ARCH_7A__ /* XXX */
+	iUseReverb = 0;
+	iUseInterpolation = 0;
+#endif
+	new_dynarec_hacks = 0;
+	cycle_multiplier = 200;
+
+	in_type1 = PSE_PAD_TYPE_STANDARD;
+	in_type2 = PSE_PAD_TYPE_STANDARD;
+}
+
 static void check_memcards(void)
 {
 	char buf[MAXPATHLEN];
@@ -206,6 +245,10 @@ void do_emu_action(void)
 				snprintf(hud_msg, sizeof(hud_msg), "SCREENSHOT TAKEN");
 			break;
 		}
+	case SACTION_VOLUME_UP:
+	case SACTION_VOLUME_DOWN:
+		plat_step_volume(emu_action == SACTION_VOLUME_UP);
+		return;
 	default:
 		return;
 	}
@@ -221,19 +264,6 @@ do_state_slot:
 
 int main(int argc, char *argv[])
 {
-	void *tmp;
-
-	tmp = dlopen("/lib/libdl.so.2", RTLD_LAZY);
-	if (tmp == NULL)
-		tmp = dlopen("/lib32/libdl.so.2", RTLD_LAZY);
-	if (tmp != NULL)
-		real_getenv = dlsym(tmp, "getenv");
-	if (real_getenv == NULL) {
-		fprintf(stderr, "%s\n", dlerror());
-		return 1;
-	}
-	dlclose(tmp);
-
 	// what is the name of the config file?
 	// it may be redefined by -cfg on the command line
 	strcpy(cfgfile_basename, "pcsx.cfg");
@@ -245,6 +275,7 @@ int main(int argc, char *argv[])
 
 	CheckSubDir();
 	set_default_paths();
+	emu_set_default_config();
 	check_memcards();
 	strcpy(Config.Bios, "HLE");
 
@@ -327,10 +358,12 @@ int main(int argc, char *argv[])
 
 	// frontend stuff
 	in_init();
+	in_evdev_init();
 	//in_probe();
 	plat_init();
 	menu_init(); // loads config
 	pl_init();
+	plat_rescan_inputs();
 
 	if (psxout)
 		Config.PsxOut = 1;
@@ -434,6 +467,9 @@ void SysReset() {
 	void *real_lace = GPU_updateLace;
 	GPU_updateLace = dummy_lace;
 
+	// reset can run code, timing must be set
+	pl_timing_prepare(Config.PsxType);
+
 	EmuReset();
 
 	// hmh core forgets this
@@ -658,20 +694,6 @@ void ClosePlugins() {
 	}
 }
 
-#if 1
-/* this is to avoid having to hack every plugin to stop using $HOME */
-char *getenv(const char *name)
-{
-	static char ret[8] = ".";
-
-	if (name && strcmp(name, "HOME") == 0 &&
-			((int)name >> 28) == 0) // HACK: let libs find home
-		return ret;
-
-	return real_getenv(name);
-}
-#endif
-
 /* we hook statically linked plugins here */
 static const char *builtin_plugins[] = {
 	"builtin_gpu", "builtin_spu", "builtin_cdr", "builtin_pad",
