@@ -48,6 +48,11 @@ void in_update_analogs(void)
 {
 }
 
+void *plat_prepare_screenshot(int *w, int *h, int *bpp)
+{
+	return 0;
+}
+
 int plat_rescan_inputs(void)
 {
 	return -1;
@@ -67,3 +72,7 @@ int plat_get_bat_capacity(void)
 {
 	return -1;
 }
+
+void plat_step_volume(int is_up)
+{
+}
