@@ -116,6 +116,10 @@ void plat_video_menu_leave(void)
 		fprintf(stderr, "warning: vout_fbdev_resize failed\n");
 }
 
+void plat_step_volume(int is_up)
+{
+}
+
 void plat_init(void)
 {
 	const char *main_fb_name, *layer_fb_name;
@@ -173,9 +177,6 @@ void plat_init(void)
 	}
 	g_menubg_ptr = temp_frame;
 
-	// hmh
-	plat_rescan_inputs();
-
 	return;
 
 fail1:
