@@ -27,8 +27,11 @@
 // Dma3   in CdRom.c
 
 void spuInterrupt() {
-	HW_DMA4_CHCR &= SWAP32(~0x01000000);
-	DMA_INTERRUPT(4);
+	if (HW_DMA4_CHCR & SWAP32(0x01000000))
+	{
+		HW_DMA4_CHCR &= SWAP32(~0x01000000);
+		DMA_INTERRUPT(4);
+	}
 }
 
 void psxDma4(u32 madr, u32 bcr, u32 chcr) { // SPU
@@ -167,8 +170,9 @@ void psxDma2(u32 madr, u32 bcr, u32 chcr) { // GPU
 			PSXDMA_LOG("*** DMA 2 - GPU dma chain *** %lx addr = %lx size = %lx\n", chcr, madr, bcr);
 #endif
 
-			size = gpuDmaChainSize(madr);
-			GPU_dmaChain((u32 *)psxM, madr & 0x1fffff);
+			size = GPU_dmaChain((u32 *)psxM, madr & 0x1fffff);
+			if ((int)size <= 0)
+				size = gpuDmaChainSize(madr);
 			
 			// Tekken 3 = use 1.0 only (not 1.5x)
 
@@ -191,8 +195,11 @@ void psxDma2(u32 madr, u32 bcr, u32 chcr) { // GPU
 }
 
 void gpuInterrupt() {
-	HW_DMA2_CHCR &= SWAP32(~0x01000000);
-	DMA_INTERRUPT(2);
+	if (HW_DMA2_CHCR & SWAP32(0x01000000))
+	{
+		HW_DMA2_CHCR &= SWAP32(~0x01000000);
+		DMA_INTERRUPT(2);
+	}
 }
 
 void psxDma6(u32 madr, u32 bcr, u32 chcr) {
@@ -238,6 +245,9 @@ void psxDma6(u32 madr, u32 bcr, u32 chcr) {
 
 void gpuotcInterrupt()
 {
-	HW_DMA6_CHCR &= SWAP32(~0x01000000);
-	DMA_INTERRUPT(6);
+	if (HW_DMA6_CHCR & SWAP32(0x01000000))
+	{
+		HW_DMA6_CHCR &= SWAP32(~0x01000000);
+		DMA_INTERRUPT(6);
+	}
 }
