@@ -6,14 +6,10 @@
 #include <hildon/hildon.h>
 #include "plugin_lib.h"
 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include <X11/keysym.h>
-#include <X11/XKBlib.h>
-
 #include "main.h"
 #include "../libpcsxcore/psemu_plugin_defs.h"
 #include "common/readpng.h"
+#include "maemo_common.h"
 
 #define X_RES           800
 #define Y_RES           480
@@ -28,12 +24,8 @@ static GdkImage *image;
 static HildonAnimationActor *actor;
 static GtkWidget *window, *drawing;
 
-int keystate;
-int pl_buf_w=0,pl_buf_h=0;
-extern int g_opts;
-int sens,y_def;
-extern char * file_name;
-int in_keystate, in_a1[2], in_a2[2];
+static int pl_buf_w, pl_buf_h;
+static int sens, y_def;
 static int keymap[65536];
 
 // map psx4m compatible keymap to PSX keys
@@ -95,26 +87,24 @@ window_key_proxy(GtkWidget *widget,
 			break;
 		case 19:
 			if (event->type == GDK_KEY_PRESS)
-		{
+			{
 				emu_save_state(state_slot);
 				char buf[MAXPATHLEN];
 				sprintf (buf,"/opt/maemo/usr/games/screenshots%s.%3.3d",file_name,state_slot);
 				writepng(buf, image->mem, pl_buf_w,pl_buf_h);
-
+			}
 			return;
-	}
 		case 20:
 			if (event->type == GDK_KEY_PRESS)
-
 				emu_load_state(state_slot);
 			return;
 		case 21:
 			if (event->type == GDK_KEY_PRESS)
-			state_slot=(state_slot<9)?state_slot+1:0;
+				state_slot=(state_slot<9)?state_slot+1:0;
 			return;
 		case 22:
 			if (event->type == GDK_KEY_PRESS)
-			state_slot=(state_slot>0)?state_slot-1:8;
+				state_slot=(state_slot>0)?state_slot-1:8;
 			return;
 	}
 
@@ -157,13 +147,13 @@ void maemo_init(int *argc, char ***argv)
 	
 	pFile = fopen("/opt/psx4m/config", "r");
 	if (NULL != pFile) {
-	fscanf(pFile, "%d %d",&sens,&y_def);
-	fclose(pFile);
-	}else{
-	sens=150;
-	y_def=500; //near 45 degrees =)
+		fscanf(pFile, "%d %d",&sens,&y_def);
+		fclose(pFile);
+	} else {
+		sens=150;
+		y_def=500; //near 45 degrees =)
 	}
-	
+
 	gtk_init (argc, argv);
 
 	window = hildon_stackable_window_new ();
@@ -179,10 +169,10 @@ void maemo_init(int *argc, char ***argv)
 				GDK_BUTTON_PRESS_MASK | GDK_BUTTON_RELEASE_MASK);
 
 	actor = HILDON_ANIMATION_ACTOR (hildon_animation_actor_new());
-if (g_opts & 2)
-	hildon_animation_actor_set_position (actor, 0, 0 );
-else
-	hildon_animation_actor_set_position (actor, (X_RES - D_WIDTH)/2, (Y_RES - D_HEIGHT)/2 );
+	if (g_maemo_opts & 2)
+		hildon_animation_actor_set_position (actor, 0, 0 );
+	else
+		hildon_animation_actor_set_position (actor, (X_RES - D_WIDTH)/2, (Y_RES - D_HEIGHT)/2 );
 	hildon_animation_actor_set_parent (actor, GTK_WINDOW (window));
 
 	drawing = gtk_image_new ();
@@ -210,16 +200,16 @@ void *hildon_set_mode(int w, int h)
 	gtk_image_set_from_image (GTK_IMAGE(drawing), image, NULL);
 
 	gtk_window_resize (GTK_WINDOW (actor), w, h);
-if (g_opts & 2)
-	hildon_animation_actor_set_scale (actor,
+	if (g_maemo_opts & 2)
+		hildon_animation_actor_set_scale (actor,
 				(gdouble)800 / (gdouble)w,
 				(gdouble)480 / (gdouble)h
-	);
-else
+				);
+	else
 		hildon_animation_actor_set_scale (actor,
 				(gdouble)D_WIDTH / (gdouble)w,
 				(gdouble)D_HEIGHT / (gdouble)h
-	);
+				);
 	pl_buf_w=w;pl_buf_h=h;
 	return pl_vout_buf;
 }
@@ -227,25 +217,26 @@ else
 void *hildon_flip(void)
 {
 	gtk_widget_queue_draw (drawing);
-	
-// process accelometer
-	if (g_opts&4){
-	int x, y, z;
-	FILE* f = fopen( "/sys/class/i2c-adapter/i2c-3/3-001d/coord", "r" );
-if( !f ) {printf ("err in accel");return;}
-	fscanf( f, "%d %d %d", &x, &y, &z );
-	fclose( f );
 
-		if( x > sens ) keystate |= 1 << DKEY_LEFT;
-		else if( x < -sens ) keystate |= 1 << DKEY_RIGHT;
-		else {keystate &= ~(1 << DKEY_LEFT);keystate &= ~(1 << DKEY_RIGHT);}
-		
+	// process accelometer
+	if (g_maemo_opts & 4) {
+		int x, y, z;
+		FILE* f = fopen( "/sys/class/i2c-adapter/i2c-3/3-001d/coord", "r" );
+		if( !f ) {printf ("err in accel"); exit(1);}
+		fscanf( f, "%d %d %d", &x, &y, &z );
+		fclose( f );
+
+		if( x > sens ) in_keystate |= 1 << DKEY_LEFT;
+		else if( x < -sens ) in_keystate |= 1 << DKEY_RIGHT;
+		else {in_keystate &= ~(1 << DKEY_LEFT);in_keystate &= ~(1 << DKEY_RIGHT);}
+
 		y+=y_def;
-		if( y > sens )keystate |= 1 << DKEY_UP;
-		else if( y < -sens ) keystate |= 1 << DKEY_DOWN; 
-		else {keystate &= ~(1 << DKEY_DOWN);keystate &= ~(1 << DKEY_UP);}
+		if( y > sens )in_keystate |= 1 << DKEY_UP;
+		else if( y < -sens ) in_keystate |= 1 << DKEY_DOWN; 
+		else {in_keystate &= ~(1 << DKEY_DOWN);in_keystate &= ~(1 << DKEY_UP);}
+
+	}
 
-}
 	/* process GTK+ events */
 	while (gtk_events_pending())
 		gtk_main_iteration();
@@ -262,3 +253,12 @@ void menu_notify_mode_change(int w, int h, int bpp)
 {
 }
 
+void *plat_prepare_screenshot(int *w, int *h, int *bpp)
+{
+	return NULL;
+}
+
+void plat_step_volume(int is_up)
+{
+}
+
